# MailParser.pm
#
# A module for parsing "simple mails" and GnuPG signed mails.
#
# author:  Max-Gerd Retzlaff <m.retzlaff@gmx.net>
#
#          MIME::Parser stuff in SignedMIMEMail::get_content()
#             by Reyk Floeter <reyk@synack.de>
#
# date:    Date: 2002/01/25 01:30:00 CET
# version: Revision: pre-alpha-nearly-untested-release 0.01
#
# This application is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version. 
#
# This application is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# 02111 USA.  

package MailParser;

use HelpfulFunctions; 

#use Exporter;
#@ISA = ('Exporter');
#@EXPORT = ( );

my $SimpleMailAllowed = 'yes';

sub new {
    my ($pkg, $text) = @_;
    return (bless \$text, $pkg);
}

sub AllowSimpleMail {
    $SimpleMailAllowed = 'yes';
}

sub ForbidSimpleMail {
    $SimpleMailAllowed = 'no';
}

sub autodetectingnew {
    my ($pkg, $text) = @_;

    my $content_type = get_header_field(\$text, 'Content-Type');
    if ( $content_type =~ /text\/plain;/ ) {
	# SimpleMail
	if ( $SimpleMailAllowed eq 'yes') {
	    return ( new('SimpleMail', $text) );
	} else {
	    mdie(0, "ERROR: SimpleMail is not allowed.\n");
	}
    } elsif ( $content_type =~ /multipart\/signed;/ ) {
	# SignedMIMEMail
	return ( new('SignedMIMEMail', $text) );
    } else {
	mdie(0, "ERROR: Can't parse Content-Type: $content_type.\n");
    }
}

sub get_header {
    my $text = ${$_[0]}; #no shift
    my ($header, undef) = split(/\n\n/, $text, 2);
    return $header;
}
# using shift you could change the mailtext in get_header():
#sub get_header {
#    my $pkg = shift;
#    my ($header, $body) = split(/\n\n/, $$pkg, 2);
#    $$pkg = 'No more lemmings!';
#    return $header;
#}

sub get_header_field {
    my $text = ${ $_[0] };
    my $field = $_[1];

    my @header = split('\n', get_header(\$text) );
    # my $hit = 0;
    my $value = '';
    foreach $line (@header) {
	if ( $line =~ /^$field:\s(.*?)$/i ) {
	    # $hit = 1;
	    $value = $1;
	}
    }
    return $value;
}

sub get_sender {
    my $text = ${ $_[0] };

    my $from = get_header_field(\$text, 'From' );

    if ($from eq '') { # no "From:" at all
	return 'anonymous';
    } elsif ( $from =~ m/(.*?)\s*?\<(.*?)\>/ ) { # John Doe <jdoe@no.where>
	return "<a href=\"mailto:$2\">$1</a>";
    } elsif ( $from =~ m/(.*?)\s*?\((.*?)\)/ ) { # jdoe@no.where (John Doe)
	return "<a href=\"mailto:$1\">$2</a>";
    } elsif ( $from =~ m/([^\s]*\@[^\s]*)/ ) { # blabla jdoe@no.where blabla
	return "<a href=\"mailto:$1\">$1</a>";
    } else { # can't recognise format, just returning
	return $from;
    }
}

sub get_mid {
    my $text = ${ $_[0] };

    my $mid = get_header_field(\$text, 'Message-ID' );
    $mid =~ s/<(.*)>/$1/;
    $mid =~ s/\@/-at-/;
    return $mid;
}

# obsolete -> now using parse_subject()
#
#sub get_type {
#    my $text = ${ $_[0] };
#
#    my $subject = get_header_field(\$text, 'Subject' );
#    (my $type = $subject )  =~ s/^(?:\[.*?\]\s)?(\w*):\s.*/$1/;
#    return $type;
#}
#
#sub get_title {
#    my $text = ${ $_[0] };
#
#    my $subject = get_header_field(\$text, 'Subject' );
#    (my $title = $subject ) =~ s/^(?:\[.*?\]\s)?\w*:\s(.*)/$1/;
#    return $title;
#}

sub parse_subject {
    my $text = ${ $_[0] };

    my $subject = get_header_field(\$text, 'Subject' );

    if ( $subject =~ m/^(?:Re:\s*)?(?:\[[^\[]*?\]\s+)?(?:(\w*):\s+)?(\w*):(?:\s+\[([^\[]*?)\])?\s+(.*)/ ) {
	# $1: command, $2: type, $3: mid, $4 title

	my ($command, $type, $mid, $title) = ($1, $2, $3, $4);
	if ( !defined($command) ){
	    $command = 'ADD';
	}
	if ( ($command eq 'REPLACE' or $command eq 'DELETE') and !defined($mid) ) {
	    mdie(0, "ERROR: You have to supply a message ID if you invoke DELETE or REPLACE.\n");
	}
	if ( $command eq 'REPLACE' or $command eq 'DELETE') {
	    return ($command, $type, $mid, $title);
	} else {
	    return ($command, $type, '', $title);
	}
    } else {
	mdie(0, "ERROR: Can't parse Subject: $subject.\n");
    }
}

sub replace_subject_and_mid { # CHANGES SUPPLIED TEXT (that is not nice...)
    my ($text_r, $command, $type, $mid, $title) = @_;

    my $new_subject = "ADD: $type: $title";
    my $current_subject = get_header_field($text_r, 'Subject' );
    if ($new_subject ne $current_subject ) {
	$$text_r =~ s/(\n)(Subject:\s.*?\n)/$1X-Original-$2/gi;
	$$text_r =~ s/(\n\n)/\nSubject: $new_subject$1/; # not s///g!
    }

    if ( $mid ne get_mid($text_r) ) {
	$$text_r =~ s/(\n)(Message-ID:\s.*?\n)/$1X-Original-$2/gi;
	$mid =~ s/-at-/\@/;
	$$text_r =~ s/(\n\n)/\nMessage-ID: <$mid>$1/; # not s///g!
    }

    return 0;
}
    
##################################################
package SimpleMail;
@ISA = ('MailParser'); # inherits from MailParser

sub get_content {
    my $text = ${$_[0]}; #no shift;

    my (undef, $content) =  split/\n\n/, $text, 2;
    return $content;
}

##################################################
package SignedMIMEMail;
@ISA = ('MailParser'); # inherits from MailParser

sub get_content {
    my $text = ${$_[0]}; #no shift;
  
    #use MIME::Parser;
    use Parser;
    use GnuPGverify;
    
    my $parser = new MIME::Parser;
    # dirty hack: decode_body() requires a patched MIME::Parser
    $parser->decode_body(0);
    $parser->output_under("/tmp");
    $parser->extract_nested_messages(0);
    my $entity = $parser->parse_data($text);
    mdie(0, "ERROR: No RFC822 signature found.\n") if (!$entity->is_multipart());
    my $content_part = $entity->parts(0)->bodyhandle->as_string();
    my $gpg_signature = $entity->parts(1)->as_string();
    #my $mail_type = MailParser::get_mail_type(\$text);
    my (undef, $mail_type, undef, undef) = MailParser::parse_subject(\$text);

    GnuPGverify->check_signature( $mail_type, $content_part, $gpg_signature);

    # Isn't it stupid to create a new MIME::Parser object,
    #  only to strip of the leading Content-*-lines? Uhm, later...
    my $parser2 = new MIME::Parser;
    my $entity2 = $parser2->parse_data($content_part);
    # Remove any mail signature ("^--\s" stuff) within 15 lines of the end
    $entity2->remove_sig(15);
    my $content = $entity2->bodyhandle->as_string();

    return $content;
}

return 1; # Last executing statement in file must be
          # non-zero, to indicate succesful loading
