# generateHTML.pm
#
# A skinnable module for generating HTML pages.
#
# author:  Max-Gerd Retzlaff <m.retzlaff@gmx.net>
# date:    Date: 2002/01/25 01:30:00 CET
# version: Revision: pre-alpha-nearly-untested-release 0.01
#
# This application is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version. 
#
# This application is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# 02111 USA.  

package generateHTML;

use HelpfulFunctions; 

$skin_version='generateHTML skin version 0.1';

#use Exporter;
#@ISA = ('Exporter');
#@EXPORT = ();

@methods = ( 'page_begin',
	     'page_end',
	     'index',
	     'content',
	     'maintitle',
	     'title',
	     'footer' );
$www_dir = '';

# default
$skin{page_begin} = "<html><head><title>[MWIKI-WILDCARD]</title></head><body>\n<table width=\"100%\" border=\"1\"><tr>";
$skin{page_end} = "</tr></table>\n[MWIKI-WILDCARD]</body></html>";
$skin{index} = "<!--index begin--><td valign=\"top\"width=\"15%\">\n[MWIKI-WILDCARD]\n</td><!--index end-->";
$skin{content} = "<!--content begin--><td valign=\"top\">\n[MWIKI-WILDCARD]</td><!--content end-->";
$skin{maintitle} = " <H1> [MWIKI-WILDCARD] </h1>";
$skin{title} = "<h2>[MWIKI-WILDCARD]</h2>";
$skin{footer} = "[MWIKI-WILDCARD]";

sub set_www_dir {
    my ($pkg, $dir) = @_;
    $www_dir = $dir;
}

sub read_skin {
    my ($pkg, $filename) = @_;
    {
	local $/ = "[MWIKI-SEPERATOR]\n";
	open(SOURCE, "< $filename") or 
	    mdie(1, "ERROR: Can't open file \"$filename\" for reading.\n");

	my $header_part = <SOURCE>; # strip of header (incl. comments)
	chomp($header_part);
	(my $file_version = $header_part ) =~ s/^(.*?)\n.*$/$1/s;

	if ( $skin_version ne $file_version ) {
	    mdie(1, "ERROR: \"$filename\" is not a \"$skin_version\"-file.\n");
	} else {
	    foreach $record (<SOURCE>) {
		chomp($record);
		my @skin = split("\n", $record );
		my $method = shift(@skin);
		$skin{$method} = join( "\n", @skin );
	        # print STDERR "+".$method."+++".join("\n",@skin);
            }
        }
	close(SOURCE);
    }
    return 0;
}

sub AUTOLOAD {
    my ($pkg, $text) = @_;
    $text = '' unless ( defined $text );
    
    return if ( $AUTOLOAD =~ /::DESTROY$/ );

    (my $subroutine = $AUTOLOAD ) =~ s/^.*:://;
    foreach my $method (@methods) {
	if ( $method eq $subroutine ) {
	    (my $output = $skin{$subroutine} ) =~ s/\[MWIKI\-WILDCARD\]/$text/g;
 	    $output =~ s/\[MWIKI\-WWW\-DIR\]/$www_dir/g;
	    return $output."\n";
        }
    }
    mdie(1, "ERROR: Don't support method \"$subroutine\".\n");
    return 1;
}

sub whole_page {
    my ($pkg, $page_title, $main_title, $index_title, $index,
	$content_title, $content, $footer) = @_;

    return generateHTML->page_begin($page_title).
	generateHTML->maintitle($main_title).
	generateHTML->index(generateHTML->title($index_title).$index).
	generateHTML->content( generateHTML->title($content_title).$content).
	generateHTML->page_end( generateHTML->footer($footer) );
}

return 1; # Last executing statement in file must be
          # non-zero, to indicate succesful loading
